@@ -7,10 +7,10 @@ import posixpath
 import threading
 from collections import defaultdict
 from contextlib import contextmanager
+from itertools import chain
 
 from fsspec.spec import AbstractFileSystem
 from funcy import cached_property, retry, wrap_prop, wrap_with
-from funcy.py3 import cat
 from tqdm.utils import CallbackIOWrapper
 
 from pydrive2.drive import GoogleDrive
@@ -245,31 +245,25 @@ class GDriveFileSystem(AbstractFileSystem):
     @wrap_prop(threading.RLock())
     @cached_property
     def _ids_cache(self):
-        cache = {
-            "dirs": defaultdict(list),
-            "ids": {},
-            "root_id": self._get_item_id(
-                self.path,
-                use_cache=False,
-                hint="Confirm the directory exists and you can access it.",
-            ),
-        }
+        cache = {"dirs": defaultdict(list), "ids": {}}
 
-        self._cache_path_id(self.base, cache["root_id"], cache=cache)
+        base_item_ids = self._path_to_item_ids(self.base, use_cache=False)
+        if not base_item_ids:
+            raise FileNotFoundError(
+                errno.ENOENT,
+                os.strerror(errno.ENOENT),
+                f"Confirm {self.path} exists and you can access it",
+            )
 
-        for item in self._gdrive_list(
-            "'{}' in parents and trashed=false".format(cache["root_id"])
-        ):
-            item_path = posixpath.join(self.base, item["title"])
-            self._cache_path_id(item_path, item["id"], cache=cache)
+        self._cache_path_id(self.base, *base_item_ids, cache=cache)
 
         return cache
 
     def _cache_path_id(self, path, *item_ids, cache=None):
         cache = cache or self._ids_cache
         for item_id in item_ids:
-            cache["dirs"][path].append(item_id)
             cache["ids"][item_id] = path
+            cache["dirs"][path].append(item_id)
 
     @cached_property
     def _list_params(self):
@@ -314,7 +308,7 @@ class GDriveFileSystem(AbstractFileSystem):
         get_list = _gdrive_retry(lambda: next(file_list, None))
 
         # Fetch pages until None is received, lazily flatten the thing.
-        return cat(iter(get_list, None))
+        return chain.from_iterable(iter(get_list, None))
 
     def _gdrive_list_ids(self, query_ids):
         query = " or ".join(
@@ -323,7 +317,9 @@ class GDriveFileSystem(AbstractFileSystem):
         query = f"({query}) and trashed=false"
         return self._gdrive_list(query)
 
-    def _get_remote_item_ids(self, parent_ids, title):
+    def _get_remote_item_ids(
+        self, parent_ids, parent_path, title, use_cache=True
+    ):
         if not parent_ids:
             return None
         query = "trashed=false and ({})".format(
@@ -333,13 +329,19 @@ class GDriveFileSystem(AbstractFileSystem):
         )
         query += " and title='{}'".format(title.replace("'", "\\'"))
 
-        # GDrive list API is case insensitive, we need to compare
-        # all results and pick the ones with the right title
-        return [
-            item["id"]
-            for item in self._gdrive_list(query)
-            if item["title"] == title
-        ]
+        res = []
+        for item in self._gdrive_list(query):
+            # GDrive list API is case insensitive, we need to compare
+            # all results and pick the ones with the right title
+            if item["title"] == title:
+                res.append(item["id"])
+
+                if item["mimeType"] == FOLDER_MIME_TYPE and use_cache:
+                    self._cache_path_id(
+                        posixpath.join(parent_path, item["title"]), item["id"]
+                    )
+
+        return res
 
     def _get_cached_item_ids(self, path, use_cache):
         if not path:
@@ -355,7 +357,9 @@ class GDriveFileSystem(AbstractFileSystem):
 
         parent_path, title = posixpath.split(path)
         parent_ids = self._path_to_item_ids(parent_path, create, use_cache)
-        item_ids = self._get_remote_item_ids(parent_ids, title)
+        item_ids = self._get_remote_item_ids(
+            parent_ids, parent_path, title, use_cache
+        )
         if item_ids:
             return item_ids
 
@@ -363,7 +367,7 @@ class GDriveFileSystem(AbstractFileSystem):
             [self._create_dir(min(parent_ids), title, path)] if create else []
         )
 
-    def _get_item_id(self, path, create=False, use_cache=True, hint=None):
+    def _get_item_id(self, path, create=False, use_cache=True):
         bucket, base = self.split_path(path)
         assert bucket == self.root
 
@@ -372,9 +376,7 @@ class GDriveFileSystem(AbstractFileSystem):
             return min(item_ids)
 
         assert not create
-        raise FileNotFoundError(
-            errno.ENOENT, os.strerror(errno.ENOENT), hint or path
-        )
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), path)
 
     @_gdrive_retry
     def _gdrive_create_dir(self, parent_id, title):
@@ -392,9 +394,7 @@ class GDriveFileSystem(AbstractFileSystem):
             return cached[0]
 
         item = self._gdrive_create_dir(parent_id, title)
-
-        if parent_id == self._ids_cache["root_id"]:
-            self._cache_path_id(remote_path, item["id"])
+        self._cache_path_id(remote_path, item["id"])
 
         return item["id"]
 
@@ -426,15 +426,13 @@ class GDriveFileSystem(AbstractFileSystem):
 
     def ls(self, path, detail=False):
         bucket, base = self.split_path(path)
+        assert bucket == self.root
 
-        cached = base in self._ids_cache["dirs"]
-        if cached:
-            dir_ids = self._ids_cache["dirs"][base]
-        else:
-            dir_ids = self._path_to_item_ids(base)
-
+        dir_ids = self._path_to_item_ids(base)
         if not dir_ids:
-            return None
+            raise FileNotFoundError(
+                errno.ENOENT, os.strerror(errno.ENOENT), path
+            )
 
         root_path = posixpath.join(bucket, base)
         contents = []
@@ -449,18 +447,16 @@ class GDriveFileSystem(AbstractFileSystem):
                     }
                 )
             else:
+                size = item.get("fileSize")
                 contents.append(
                     {
                         "type": "file",
                         "name": item_path,
-                        "size": int(item["fileSize"]),
+                        "size": int(size) if size is not None else size,
                         "checksum": item.get("md5Checksum"),
                     }
                 )
 
-        if not cached:
-            self._cache_path_id(root_path, *dir_ids)
-
         if detail:
             return contents
         else:
@@ -468,9 +464,14 @@ class GDriveFileSystem(AbstractFileSystem):
 
     def find(self, path, detail=False, **kwargs):
         bucket, base = self.split_path(path)
+        assert bucket == self.root
+
+        # Make sure the base path is cached and dir_ids below has some
+        # dirs revelant to this call
+        self._path_to_item_ids(base)
 
-        seen_paths = set()
         dir_ids = [self._ids_cache["ids"].copy()]
+        seen_paths = set()
         contents = []
         while dir_ids:
             query_ids = {
@@ -493,12 +494,12 @@ class GDriveFileSystem(AbstractFileSystem):
                     new_query_ids[item["id"]] = item_path
                     self._cache_path_id(item_path, item["id"])
                     continue
-
+                size = item.get("fileSize")
                 contents.append(
                     {
                         "name": posixpath.join(bucket, item_path),
                         "type": "file",
-                        "size": int(item["fileSize"]),
+                        "size": int(size) if size is not None else size,
                         "checksum": item.get("md5Checksum"),
                     }
                 )
@@ -544,7 +545,6 @@ class GDriveFileSystem(AbstractFileSystem):
 
     @_gdrive_retry
     def mv(self, path1, path2, maxdepth=None, **kwargs):
-
         if maxdepth is not None:
             raise NotImplementedError("Max depth move is not supported")
 
