@@ -1,6 +1,9 @@
 import filecmp
+import json
 import os
 import unittest
+from unittest.mock import MagicMock
+import httplib2
 import pytest
 import sys
 from io import BytesIO
@@ -25,6 +28,59 @@ from pydrive2.test.test_util import (
 )
 
 
+def auth_with_resource_key_mock() -> GoogleAuth:
+    """
+    Create GoogleAuth with mocked inner httplib2.Http simulating need
+    for resourceKey header.
+    """
+    http_mock = MagicMock()
+
+    def resource_key_request(
+        uri,
+        method="GET",
+        body=None,
+        headers=None,
+        redirections=1,
+        connection_type=None,
+    ):
+        """httplib2.Http.request mock."""
+        # If "media" is not requested, it means we expect metadata
+        fetch_meta_data = "alt=media" not in uri
+        if (
+            headers
+            and "X-Goog-Drive-Resource-Keys" in headers
+            and headers["X-Goog-Drive-Resource-Keys"]
+            == "0BxphPoRgwhnodHNjS3JESnFNS1E/0-vjzOveuin3fnf4LUlfsD3A"
+        ):
+            if fetch_meta_data:
+                # Fake meta data query response
+                content = json.dumps({"title": "N48E012.zip"}).encode()
+            else:
+                # Fake file content query response
+                content = b"some content"
+            return (
+                httplib2.Response(
+                    {"status": "200", "content-length": str(len(content))}
+                ),
+                content,
+            )
+        # Simulate 404 response for file not found; body must be valid error JSON
+        return (
+            httplib2.Response({"status": "404"}),
+            json.dumps({"error": {"code": 404}}).encode(),
+        )
+
+    http_mock.request.side_effect = resource_key_request
+    ga = GoogleAuth(
+        settings_file_path(
+            "default.yaml", os.path.join(os.path.dirname(__file__), "")
+        )
+    )
+    ga.thread_local.http = http_mock
+    ga.ServiceAuth()
+    return ga
+
+
 class GoogleDriveFileTest(unittest.TestCase):
     """Tests basic file operations of files.GoogleDriveFile.
     Upload and download of contents and metadata, and thread-safety checks.
@@ -129,13 +185,13 @@ class GoogleDriveFileTest(unittest.TestCase):
     def test_Files_Insert_Content_File(self):
         drive = GoogleDrive(self.ga)
         file1 = drive.CreateFile()
-        filename = self.getTempFile("filecontent")
-        file1["title"] = filename
         contentFile = self.getTempFile("actual_content", "some string")
         file1.SetContentFile(contentFile)
         pydrive_retry(file1.Upload)  # Files.insert
 
-        self.assertEqual(file1.metadata["title"], filename)
+        self.assertEqual(
+            file1.metadata["title"], os.path.basename(contentFile)
+        )
         pydrive_retry(
             file1.FetchContent
         )  # Force download and double check content.
@@ -264,6 +320,26 @@ class GoogleDriveFileTest(unittest.TestCase):
 
         self.DeleteUploadedFiles(drive, [file1["id"]])
 
+    def test_Files_Update_File_Keeps_Filename(self):
+        # Tests https://github.com/iterative/PyDrive2/issues/272
+        drive = GoogleDrive(self.ga)
+        file1 = drive.CreateFile()
+        filename = self.getTempFile("preupdatetestfile")
+        contentFile = self.getTempFile("actual_content", "some string")
+        contentFile2 = self.getTempFile("actual_content_2", "some string")
+
+        file1["title"] = filename
+        file1.SetContentFile(contentFile)
+        pydrive_retry(file1.Upload)  # Files.insert
+        self.assertEqual(file1.metadata["title"], filename)
+
+        same_file = drive.CreateFile({"id": file1["id"]})
+        same_file.SetContentFile(contentFile2)
+        pydrive_retry(same_file.Upload)  # Files.update
+        self.assertEqual(same_file.metadata["title"], filename)
+
+        self.DeleteUploadedFiles(drive, [file1["id"]])
+
     def test_Files_Update_By_Id(self):
         drive = GoogleDrive(self.ga)
         file1 = drive.CreateFile()
@@ -336,6 +412,128 @@ class GoogleDriveFileTest(unittest.TestCase):
 
         self.DeleteUploadedFiles(drive, [file1["id"]])
 
+    def test_Files_Get_Content_Buffer_resourceKey_missing(self):
+        """404 expected for file secured with resourceKey when not provided."""
+
+        ga = auth_with_resource_key_mock()
+
+        drive = GoogleDrive(ga)
+        file1 = drive.CreateFile(
+            {
+                "id": "0BxphPoRgwhnodHNjS3JESnFNS1E",
+            }
+        )
+        with self.assertRaisesRegex(
+            ApiRequestError, "HttpError 404 when requesting"
+        ):
+            pydrive_retry(file1.GetContentIOBuffer)
+
+    def test_Files_Get_Content_Buffer_resourceKey(self):
+        """End to end scenario with real file."""
+        ga = auth_with_resource_key_mock()
+        drive = GoogleDrive(ga)
+        file1 = drive.CreateFile(
+            {
+                "id": "0BxphPoRgwhnodHNjS3JESnFNS1E",
+                "resourceKey": "0-vjzOveuin3fnf4LUlfsD3A",
+            }
+        )
+
+        buffer1 = pydrive_retry(file1.GetContentIOBuffer)
+
+        self.assertEqual(len(buffer1), 12)
+
+    @pytest.mark.manual
+    def test_Files_Get_Content_Buffer_resourceKey_missing_real(self):
+        """
+        404 expected for file secured with resourceKey when not provided.
+        End to end scenario with real public file.
+        """
+        drive = GoogleDrive(self.ga)
+        file1 = drive.CreateFile(
+            {
+                "id": "0BxphPoRgwhnodHNjS3JESnFNS1E",
+            }
+        )
+        with self.assertRaisesRegex(
+            ApiRequestError, "HttpError 404 when requesting"
+        ):
+            pydrive_retry(file1.GetContentIOBuffer)
+
+    @pytest.mark.manual
+    def test_Files_Get_Content_Buffer_resourceKey_real(self):
+        """End to end scenario with real public file."""
+        drive = GoogleDrive(self.ga)
+        file1 = drive.CreateFile(
+            {
+                "id": "0BxphPoRgwhnodHNjS3JESnFNS1E",
+                "resourceKey": "0-vjzOveuin3fnf4LUlfsD3A",
+            }
+        )
+
+        buffer1 = pydrive_retry(file1.GetContentIOBuffer)
+
+        self.assertEqual(len(buffer1), 6128902)
+
+    def test_Files_Get_Content_File_resourceKey_missing(self):
+        """404 expected for file secured with resourceKey when not provided."""
+        ga = auth_with_resource_key_mock()
+        drive = GoogleDrive(ga)
+        file1 = drive.CreateFile(
+            {
+                "id": "0BxphPoRgwhnodHNjS3JESnFNS1E",
+            }
+        )
+        fileOut = self.getTempFile()
+        with self.assertRaisesRegex(
+            ApiRequestError, "HttpError 404 when requesting"
+        ):
+            pydrive_retry(file1.GetContentFile, fileOut)
+
+    def test_Files_Get_Content_File_resourceKey(self):
+        ga = auth_with_resource_key_mock()
+        drive = GoogleDrive(ga)
+        file1 = drive.CreateFile(
+            {
+                "id": "0BxphPoRgwhnodHNjS3JESnFNS1E",
+                "resourceKey": "0-vjzOveuin3fnf4LUlfsD3A",
+            }
+        )
+
+        fileOut = self.getTempFile()
+        pydrive_retry(file1.GetContentFile, fileOut)
+
+        with open(fileOut, "rb") as f:
+            self.assertEqual(len(f.read()), 12)
+
+    def test_Files_Fetch_Metadata_resourceKey_missing(self):
+        """404 expected for file secured with resourceKey when not provided."""
+        ga = auth_with_resource_key_mock()
+        drive = GoogleDrive(ga)
+        file1 = drive.CreateFile(
+            {
+                "id": "0BxphPoRgwhnodHNjS3JESnFNS1E",
+            }
+        )
+        with self.assertRaisesRegex(
+            ApiRequestError, "HttpError 404 when requesting"
+        ):
+            pydrive_retry(file1.FetchMetadata)
+
+    def test_Files_Fetch_Metadata_resourceKey(self):
+        ga = auth_with_resource_key_mock()
+        drive = GoogleDrive(ga)
+        file1 = drive.CreateFile(
+            {
+                "id": "0BxphPoRgwhnodHNjS3JESnFNS1E",
+                "resourceKey": "0-vjzOveuin3fnf4LUlfsD3A",
+            }
+        )
+
+        pydrive_retry(file1.FetchMetadata)
+
+        self.assertEqual(file1.metadata["title"], "N48E012.zip")
+
     def test_Upload_Download_Empty_File(self):
         filename = os.path.join(self.tmpdir, str(time()))
         create_file(filename, "")
@@ -578,6 +776,19 @@ class GoogleDriveFileTest(unittest.TestCase):
 
         pydrive_retry(file1.Delete)
 
+    def test_Files_Get_Revisions(self):
+        drive = GoogleDrive(self.ga)
+        file1 = drive.CreateFile()
+        pydrive_retry(file1.Upload)
+
+        self.assertFalse("revisions" in file1)
+
+        revisions = pydrive_retry(file1.GetRevisions)
+        self.assertTrue(revisions is not None)
+        self.assertTrue("revisions" in file1)
+
+        pydrive_retry(file1.Delete)
+
     def test_ApiRequestError_HttpError_Propagation(self):
         file = GoogleDrive(self.ga).CreateFile()
         pydrive_retry(file.Upload)
